@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  *  arch-tag: The Rhythmbox main entrypoint
  *
@@ -26,7 +27,6 @@
 #include <libgnome/gnome-program.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-ui-init.h>
-#include <libgnomeui/gnome-authentication-manager.h>
 #include <gtk/gtk.h>
 #include <bonobo/Bonobo.h>
 #include <bonobo/bonobo-main.h>
@@ -73,6 +73,11 @@ static gboolean playpause       = FALSE;
 static gboolean focus           = FALSE;
 static gboolean previous        = FALSE;
 static gboolean next            = FALSE;
+static gboolean volume_up       = FALSE;
+static gboolean volume_down     = FALSE;
+static gboolean toggle_mute     = FALSE;
+static gboolean rate_up         = FALSE;
+static gboolean rate_down       = FALSE;
 static gboolean shuffle         = FALSE;
 static gboolean print_play_time = FALSE;
 static gboolean print_song_length = FALSE;
@@ -86,7 +91,7 @@ main (int argc, char **argv)
 	RBShell *rb_shell;
 	char **new_argv;
 
-	const struct poptOption popt_options[] =
+	struct poptOption popt_options[] =
 	{
 		{ "print-playing",	0,  POPT_ARG_NONE,          &print_playing,                                  0, N_("Print the playing song and exit"),     NULL },
 		{ "print-playing-path",	0,  POPT_ARG_NONE,          &print_playing_path,                          0, N_("Print the playing song URI and exit"),     NULL },
@@ -99,6 +104,11 @@ main (int argc, char **argv)
 		{ "focus",	    0,  POPT_ARG_NONE,			&focus,	        	0, N_("Focus the running player"),     NULL },
 		{ "previous",		    0,  POPT_ARG_NONE,			&previous,	        	0, N_("Jump to previous song"),     NULL },
 		{ "next",		        0,  POPT_ARG_NONE,			&next,		        	0, N_("Jump to next song"),     NULL },
+		{ "volume-up",		        0,  POPT_ARG_NONE,			&volume_up,		        	0, N_("Turn up volume"),     NULL },
+		{ "volume-down",       	        0,  POPT_ARG_NONE,			&volume_down,		        	0, N_("Turn down volume"),     NULL },
+		{ "toggle-mute",		0,  POPT_ARG_NONE,			&toggle_mute,		        	0, N_("Toggle muting"),     NULL },
+		{ "rate-up",		        0,  POPT_ARG_NONE,			&rate_up,		        	0, N_("Increase rating"),     NULL },
+		{ "rate-down",       	        0,  POPT_ARG_NONE,			&rate_down,		        	0, N_("Decrease rating"),     NULL },
 		
 		{ "shuffle",		        0,  POPT_ARG_NONE,			&shuffle,		        	0, N_("Toggle shuffling"),     NULL },
 
@@ -108,6 +118,9 @@ main (int argc, char **argv)
 		{ "dry-run", 0,  POPT_ARG_NONE,          &dry_run,                             0, N_("Don't save any data permanently (implies --no-registration)"), NULL },
 		{ "rhythmdb-file", 0,  POPT_ARG_STRING,          &rhythmdb_file,                             0, N_("Path for database file to use"), NULL },
 		{ "quit",            'q',  POPT_ARG_NONE,          &quit,                                         0, N_("Quit Rhythmbox"),            NULL },
+#ifdef HAVE_GSTREAMER
+		{NULL, '\0', POPT_ARG_INCLUDE_TABLE, NULL, 0, "GStreamer", NULL},
+#endif
 		POPT_TABLEEND
 	};
 
@@ -118,6 +131,11 @@ main (int argc, char **argv)
 	new_argv[argc] = g_strdup ("--disable-sound");
 	new_argv[argc+1] = NULL;
 
+#ifdef HAVE_GSTREAMER	
+	popt_options[(sizeof(popt_options)/sizeof(popt_options[0]))-2].arg
+		= (void *) gst_init_get_popt_table ();
+#endif
+
 	gtk_set_locale ();
 	program = gnome_program_init (PACKAGE, VERSION,
 				      LIBGNOMEUI_MODULE, argc+1, new_argv,
@@ -126,7 +144,13 @@ main (int argc, char **argv)
 				      GNOME_PARAM_APP_DATADIR, DATADIR,
 				      NULL);
 
+	/* Disabled because it breaks internet radio and other things -
+	 * doing synchronous calls in the main thread causes deadlock.
+	 * This is too hard to fix right now, so we punt.
+	 */
+#if 0
 	gnome_authentication_manager_init ();
+#endif
 
 	g_random_set_seed (time(0));
 
@@ -138,7 +162,6 @@ main (int argc, char **argv)
 #endif
 
 #ifdef HAVE_GSTREAMER	
-	gst_init_with_popt_table (&argc, &argv, popt_options);
 	gst_control_init (&argc, &argv);
 #endif
 #ifdef WITH_MONKEYMEDIA
@@ -308,6 +331,45 @@ rb_handle_cmdline (char **argv, int argc,
 		return;
 	}
 
+	if (rate_up)
+	{
+		GNOME_Rhythmbox_SongInfo *song_info;
+
+		song_info = get_song_info (shell);
+		if (song_info != NULL) {
+			GNOME_Rhythmbox_setRating (shell, song_info->rating + 1, &ev);
+			CORBA_free (song_info);
+		}
+		grab_focus = FALSE;
+	}
+
+	if (rate_down)
+	{
+		GNOME_Rhythmbox_SongInfo *song_info;
+
+		song_info = get_song_info (shell);
+		if (song_info != NULL) {
+			GNOME_Rhythmbox_setRating (shell, song_info->rating - 1, &ev);
+			CORBA_free (song_info);
+		}
+		grab_focus = FALSE;
+	}
+
+        if (volume_up) {
+                GNOME_Rhythmbox_volumeUp (shell, &ev);
+		grab_focus = FALSE;
+        }
+
+        if (volume_down) {
+                GNOME_Rhythmbox_volumeDown (shell, &ev);
+		grab_focus = FALSE;
+        }
+
+        if (toggle_mute) {
+                GNOME_Rhythmbox_toggleMute (shell, &ev);
+		grab_focus = FALSE;
+        }
+
 	if (print_playing)
 	{
 		GNOME_Rhythmbox_SongInfo *song_info;
